#!/usr/bin/env bash

ARCH=$ARCH
CROSS_COMPILE=$CROSS_COMPILE
TARGETS=
HOST_ARCH=`uname -m`
# set default baseline commit id here
# e.g for klinux build: CMIT_BASE=74c827
CMIT_BASE=
CMIT_NEW=`git show --name-only --oneline HEAD 2>/dev/null | awk 'NR==1 { print $1}'`
CMIT_SAVED=`git branch | grep ^* | awk '{print $2}'`
WHITEL_F=
CONFIG_F=

trap 'do_ctrlc' INT

do_exit()
{
	git checkout $CMIT_SAVED &>/dev/null
	exit $1
}

do_ctrlc()
{
	echo_colo "Ctrl+C is captured!" red
	do_exit 0
}

echo_colo()
{
	case $2 in
		red)	echo -e "\033[31m$1\033[0m";;
		green)	echo -e "\033[32m$1\033[0m";;
		yellow)	echo -e "\033[33m$1\033[0m";;
		blue)	echo -e "\033[34m$1\033[0m";;
		*)		echo -e "\033[30m$1\033[0m";;
	esac
}

usage()
{
	echo "Usage: $0 [-a ARCH] [-w WHITELIST] [-n NEW] -b BASE -c CONFIG"
	echo "  -a, --arch	Specify ARCH"
	echo "  -b, --baseline	Baseline commit"
	echo "  -c, --config	Specify kernel config to build kernel"
	echo "  -h, --help	Help"
	echo "  -n, --new	New commit"
	echo "  -w, --whitelist	Specify whitelist file"
	echo " "
	echo "For detail instructions, please see : "
	echo "<http://pm.kylin.com/biz/task-view-17942.html>"
}

if [ x"" = x"`which dump_abi`" ]; then
	echo_colo "Please install abigail tools first!" red && do_exit 1
fi

ARGS=`getopt -o a:b:c:h::n:w: --long arch:,baseline:,config:,help::,new:,whitelist: -n '$0' -- "$@"`

if [ $? != 0 ]; then
    echo "Terminating..."
    do_exit 1
fi

eval set -- "${ARGS}"

while true; do
	case "$1" in
		-h|--help)
			usage
			do_exit 0
			;;
		-a|--arch)
			ARCH=$2
			shift 2
			;;
		-b|--baseline)
			CMIT_BASE=$2
			shift 2
			;;
		-n|--new)
			CMIT_NEW=`git show --name-only --oneline $2 2>/dev/null | awk 'NR==1 { print $1}'`
			shift 2
			;;
		-c|--config)
			CONFIG_F=$2
			shift 2
			;;
		-w|--whitelist)
			WHITEL_F=$2
			shift 2
			;;
		--)
			shift
			break
			;;
		*)
			echo "Internal error!"
			do_exit 1
			;;
	esac
done

if [ ! "$CMIT_BASE" ]; then
	usage
	do_exit 1
fi

CMIT_BASE=`git show --name-only --oneline $CMIT_BASE 2>/dev/null | awk 'NR==1 { print $1}'`

if [ x"$ARCH" = x"" ]; then
	if [ x"$CONFIG_F" != x"" ]; then
		ARCH=`echo "$CONFIG_F" | awk '{ print gensub(/arch\/(.*)\/configs\/(.*)/, "\\\\1", "g") }'`
		if [ x"$ARCH" != x"arm64" -a x"$ARCH" != x"mips" -a x"$ARCH" != x"x86" ]; then
			ARCH=$HOST_ARCH
		else
			echo_colo "=== Get ARCH=${ARCH} from config_file path <$CONFIG_F>" yellow
		fi
	else
		ARCH=$HOST_ARCH
	fi
fi

case $ARCH in
	arm64|aarch64)
		ARCH=arm64
		if [ x"$HOST_ARCH" != x"aarch64" -a x"$CROSS_COMPILE" = x"" ]; then
			CROSS_COMPILE=aarch64-linux-gnu-
		fi
		TARGETS="Image modules"
		;;
	mips|mips64el)
		ARCH=mips
		if [ x"$HOST_ARCH" != x"mips64el" -a x"$CROSS_COMPILE" = x"" ]; then
			CROSS_COMPILE=mips-linux-gnu-
		fi
		TARGETS="vmlinuz modules"
		;;
	x86|x86_64)
		ARCH=x86
		if [ x"$HOST_ARCH" != x"x86_64" -a x"$CROSS_COMPILE" = x"" ]; then
			CROSS_COMPILE=x86_64-linux-gnu-
		fi
		TARGETS="vmlinux modules"
		;;
	*)
		echo_colo "=== Unknow ARCH=${ARCH}" red && do_exit 1
		;;
esac

if [ x"$CONFIG_F" = x"" ]; then
	CONFIG_F="arch/${ARCH}/configs/kylinos_generic_defconfig"
fi

if [ ! -f $CONFIG_F ]; then
	echo_colo "=== config file <$CONFIG_F> invalid!" red && do_exit 1
fi

if [ -z "$WHITEL_F" ]; then
	if [ -f rpmbuild/SOURCES/kabi-whitelists/kabi_whitelist_${Arch} ]; then
		WHITEL_F=rpmbuild/SOURCES/kabi-whitelists/kabi_whitelist_${Arch}
	else
		echo_colo "=== rpmbuild/SOURCES/kabi-whitelists/kabi_whitelist_${Arch} not found, use parameter --whitelist to specify whitelist!" red \
			&& do_exit 1
	fi
else
	if [ ! -f $WHITEL_F ]; then
		echo_colo "=== whitelist file <$WHITEL_F> invalid!" red && do_exit 1
	fi
fi

export ARCH=${ARCH}
export CROSS_COMPILE=${CROSS_COMPILE}

LOGFILE=

run_make()
{
        KBUILD_PRESERVE=1 KBUILD_SYMTYPES=1  make $@ &>$LOGFILE
}

do_compile()
{
	echo -n "=== Checkout commit $1, "
	git checkout . &>/dev/null && git checkout $1 &>/dev/null
	if [ $? != 0 ]; then
		echo_colo "failed!" red
		echo_colo "Please try <git checkout . && git checkout $1> manually" blue
		return 1
	else
		echo_colo "done!" green
	fi

	BUILDDIR="build-${ARCH}-$1"
	echo -n "=== Create output dir $BUILDDIR, "
	mkdir $BUILDDIR &>/dev/null
	if [ -d $BUILDDIR ]; then
		echo_colo "done!" green
	else
		echo_colo "failed!" red
		return 1
	fi

	echo -n "=== Check config file <$CONFIG_F>, "
	if [ ! -f $CONFIG_F ]; then
		echo_colo "file not exist!" red
		return 1
	fi
	cp $CONFIG_F ${BUILDDIR}/.config
	sed -i 's/# CONFIG_MODVERSIONS is not set/CONFIG_MODVERSIONS=y/' ${BUILDDIR}/.config
	sed -i 's/# CONFIG_DEBUG_INFO is not set/CONFIG_DEBUG_INFO=y/' ${BUILDDIR}/.config
	echo_colo "done!" green

	LOGFILE="/tmp/build-${ARCH}-$1.log"
	echo -n "=== Build commit [$1], Log to <$LOGFILE>, "
	NPROC=$((`nproc`*2))
	run_make O=${BUILDDIR}/ olddefconfig && \
		run_make O=${BUILDDIR}/ -j${NPROC} ${TARGETS}

	if [ $? != 0 ]; then
		echo_colo "failed!" red
		return 1
	else
		echo_colo "done!" green
		rm $LOGFILE
		echo "=== Dump ABI to file <${BUILDDIR}/abi.xml>"
		dump_abi --kmi-whitelist ${WHITEL_F} \
			--linux-tree ${BUILDDIR}
		return 0
	fi
}

echo "=== Prepare to build new commit [$CMIT_NEW]"
do_compile $CMIT_NEW || do_exit 1

echo "=== Prepare to build baseline commit [$CMIT_BASE]"
do_compile $CMIT_BASE || do_exit 1

BASABI_F="build-${ARCH}-${CMIT_BASE}/abi.xml"
NEWABI_F="build-${ARCH}-${CMIT_NEW}/abi.xml"


if [ -f $BASABI_F -a -f $NEWABI_F ]; then
	diff_abi --baseline $BASABI_F --new $NEWABI_F \
       		--kmi-symbol-list ${WHITEL_F} --report report.out
	echo_colo "=== Please check ABI diff file <report.out>" yellow
fi

do_exit 0
